Package: MultiAssayExperiment
Title: Software for the integration of multi-omics experiments in
        Bioconductor
Version: 1.38.0
Authors@R: c(
    person("Marcel", "Ramos", , "marcel.ramos@sph.cuny.edu",
        c("aut", "cre"), c(ORCID = "0000-0002-3242-0582")),
    person("Martin", "Morgan", , , c("aut", "ctb")),
    person("Lori", "Shepherd", , , "ctb"),
    person("Hervé", "Pagès", , , "ctb"),
    person("Vincent J", "Carey", , , c("aut", "ctb")),
    person("Levi", "Waldron", , "lwaldron.research@gmail.com", "aut"),
    person("MultiAssay", "SIG", , "biocmultiassay@googlegroups.com", "ctb"),
    person("NCI", role = "fnd", comment = c(GrantNo. = "U24CA289073"))
    )
Description: Harmonize data management of multiple experimental assays
    performed on an overlapping set of specimens.  It provides a familiar
    Bioconductor user experience by extending concepts from
    SummarizedExperiment, supporting an open-ended mix of standard data
    classes for individual assays, and allowing subsetting by genomic
    ranges or rownames. Facilities are provided for reshaping data into
    wide and long formats for adaptability to graphing and downstream
    analysis.
License: Artistic-2.0
URL: http://waldronlab.io/MultiAssayExperiment/
BugReports: https://github.com/waldronlab/MultiAssayExperiment/issues
Depends: SummarizedExperiment, R (>= 4.5.0)
Imports: Biobase, BiocBaseUtils, BiocGenerics, DelayedArray,
        GenomicRanges, IRanges, MatrixGenerics, methods, S4Vectors,
        tidyr, utils
Suggests: BiocStyle, HDF5Array, h5mread, knitr, maftools,
        RaggedExperiment, reshape2, rmarkdown, survival, survminer,
        testthat, UpSetR
VignetteBuilder: knitr
biocViews: Infrastructure, DataRepresentation
Encoding: UTF-8
RoxygenNote: 7.3.3
Roxygen: list(markdown = TRUE)
Video: https://youtu.be/w6HWAHaDpyk, https://youtu.be/Vh0hVVUKKFM
Collate: 'ExperimentList-class.R' 'MultiAssayExperiment-class.R'
        'MatchedAssayExperiment-class.R' 'subsetBy-methods.R'
        'MultiAssayExperiment-subset.R'
        'MultiAssayExperiment-methods.R'
        'MultiAssayExperiment-helpers.R' 'MultiAssayExperiment-pkg.R'
        'MultiAssayExperimentToMAF.R' 'assay-methods.R' 'data.R'
        'hasAssay.R' 'listToMap.R' 'mapToList.R' 'prepMultiAssay.R'
        'reexports.R' 'saveHDF5MultiAssayExperiment.R' 'upsetSamples.R'
        'utilities.R'
Date: 2026-03-28
git_url: https://git.bioconductor.org/packages/MultiAssayExperiment
git_branch: RELEASE_3_23
git_last_commit: 497fcca
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:29:43 UTC; biocbuild
Author: Marcel Ramos [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3242-0582>),
  Martin Morgan [aut, ctb],
  Lori Shepherd [ctb],
  Hervé Pagès [ctb],
  Vincent J Carey [aut, ctb],
  Levi Waldron [aut],
  MultiAssay SIG [ctb],
  NCI [fnd] (GrantNo.: U24CA289073)
Maintainer: Marcel Ramos <marcel.ramos@sph.cuny.edu>
