# Generated by roxygen2: do not edit by hand

S3method(.DollarNames,MultiAssayExperiment)
export("drops<-")
export("experiments<-")
export("sampleMap<-")
export(DataFrame)
export(ExperimentList)
export(MatchedAssayExperiment)
export(MultiAssayExperiment)
export(MultiAssayExperimentToMAF)
export(anyReplicated)
export(drops)
export(experiments)
export(exportClass)
export(getWithColData)
export(hasAssay)
export(hasRowData)
export(hasRowRanges)
export(intersectByRowData)
export(intersectColumns)
export(intersectRows)
export(listToMap)
export(loadHDF5MultiAssayExperiment)
export(makeHitList)
export(mapToList)
export(mergeReplicates)
export(prepMultiAssay)
export(renameColname)
export(renamePrimary)
export(replicated)
export(replicates)
export(sampleMap)
export(saveHDF5MultiAssayExperiment)
export(showReplicated)
export(subsetByAssay)
export(subsetByColData)
export(subsetByColumn)
export(subsetByRow)
export(subsetByRowData)
export(upsetSamples)
export(wideFormat)
exportClasses(ExperimentList)
exportClasses(MatchedAssayExperiment)
exportClasses(MultiAssayExperiment)
exportMethods("$")
exportMethods("$<-")
exportMethods("[<-")
exportMethods("[[")
exportMethods("[[<-")
exportMethods("colData<-")
exportMethods("colnames<-")
exportMethods("drops<-")
exportMethods("experiments<-")
exportMethods("metadata<-")
exportMethods("names<-")
exportMethods("sampleMap<-")
exportMethods(anyReplicated)
exportMethods(assay)
exportMethods(assays)
exportMethods(coerce)
exportMethods(colData)
exportMethods(colnames)
exportMethods(complete.cases)
exportMethods(dimnames)
exportMethods(drops)
exportMethods(experiments)
exportMethods(exportClass)
exportMethods(hasRowData)
exportMethods(hasRowRanges)
exportMethods(intersectByRowData)
exportMethods(isEmpty)
exportMethods(length)
exportMethods(longForm)
exportMethods(mergeReplicates)
exportMethods(metadata)
exportMethods(names)
exportMethods(replicated)
exportMethods(replicates)
exportMethods(rownames)
exportMethods(sampleMap)
exportMethods(show)
exportMethods(showReplicated)
exportMethods(splitAssays)
exportMethods(subsetByRow)
exportMethods(updateObject)
import(BiocGenerics)
import(GenomicRanges)
import(IRanges)
import(S4Vectors)
import(SummarizedExperiment)
import(methods)
importFrom(BiocGenerics,colnames)
importFrom(BiocGenerics,longForm)
importFrom(BiocGenerics,rownames)
importFrom(MatrixGenerics,rowRanges)
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,metadata)
importFrom(SummarizedExperiment,rowData)
importFrom(methods,as)
importFrom(utils,.DollarNames)
