#' MultiAssayExperiment: Build an integrative multi-assay container
#'
#' MultiAssayExperiment allows the manipulation of related multiassay
#' datasets with partially overlapping samples, associated metadata at
#' the level of an entire study, and at the level of the "biological unit".
#' The biological unit may be a patient, plant, yeast strain, etc.
#'
#' The package hierarchy of information:
#' \itemize{
#' \item study
#' \item experiments
#' \item samples
#' }
#'
#' @aliases NULL
"_PACKAGE"
