#' @name reexports
#'
#' @title Objects exported from other packages
#'
#' @description These objects are imported from other packages. Click on the
#' function name to see their documentation.
#'
#' * [`S4Vectors::DataFrame-class`]
#'
#' @aliases NULL reexports DataFrame
#'
#' @return * `S4Vectors::DataFrame`: A `DataFrame` class object
#'
#' @examples
#' DataFrame()
#'
#' @export DataFrame
NULL
