% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reexports.R
\name{reexports}
\alias{reexports}
\alias{DataFrame}
\title{Objects exported from other packages}
\value{
\itemize{
\item \code{S4Vectors::DataFrame}: A \code{DataFrame} class object
}
}
\description{
These objects are imported from other packages. Click on the
function name to see their documentation.
\itemize{
\item \code{\link[S4Vectors:DataFrame-class]{S4Vectors::DataFrame}}
}
}
\examples{
DataFrame()

}
